/**
 mfstream - maya file string (for lack of a better name)

  This is a simple class to wrap binary io for maya files. 
  Since this is an early release, I'm happy with it just
  reading the entire file into a big buffer. Later it would be
  nice to implement this class using a smaller and smarter
  buffering system.

  Copyright 2001 Colin Stolley
*/
  

#include "mfstream.h"

// creates output file ~file
mfstream::mfstream( char* file ) {
	char* tempfile = new char[strlen(file)+2];
	strcpy( tempfile, "~" );
    strcat( tempfile, getFilename( file ) );
	init( file, tempfile );
	delete[] tempfile;
}

mfstream::mfstream( char* input, char* output ) {
	init( input, output );
}

void mfstream::init( char* input, char* output ) {
	Input_buffer_pos = 0;
	Input_buffer_size = 0;
	Output_buffer_pos = 0;
	Output_buffer_size = OUTPUT_BUFFER_SIZE;

	Input_file_name = new char[strlen(input)+1];
	Output_file_name = new char[strlen(output)+1];

	strcpy(Input_file_name, input);
	strcpy(Output_file_name, output);
	

	openInputFile( Input_file_name );
	openOutputFile( Output_file_name );

	if ( ready() ) {
		Output_buffer = new unsigned char[OUTPUT_BUFFER_SIZE];
		loadBuffer();
	}
}

void mfstream::openInputFile( char* file ) {
	
	Input_file_ptr = fopen( file, "rb" );

	if (! Input_file_ptr ) {
		printf( "ERROR: Can not open file for reading: '%s'\n", file );
		InputValid = false;
	} else { 
		InputValid = true; 
	}

}

void mfstream::openOutputFile( char* file ) {
	
	Output_file_ptr = fopen( file, "wb" );

	if (! Output_file_ptr ) {
		printf( "ERROR: Can not open file for writing: '%s'\n", file );
		OutputValid = false;
	} else {
		OutputValid = true;
	}
}


long mfstream::getFileSize() {
	// get current pos
	long cur = ftell(Input_file_ptr);

	fseek( Input_file_ptr, 0L, SEEK_SET );
	long pos = ftell(Input_file_ptr);
	fseek( Input_file_ptr, 0L, SEEK_END );
	long end = ftell(Input_file_ptr);

	// now put ptr back to original location
	fseek( Input_file_ptr, cur, 0 );

	return (end-pos);
}

// read the whole blasted file into a big buffer
int mfstream::loadBuffer() {

	Input_buffer_size = getFileSize();

	Input_buffer = new unsigned char[Input_buffer_size];
	fseek( Input_file_ptr, 0, SEEK_SET );

	long bytes_read = fread( Input_buffer, 1, Input_buffer_size, Input_file_ptr );
#ifdef _DEBUG
	printf("File size: %d\tBytes Read: %d\n", Input_buffer_size, bytes_read );
#endif

	if (! bytes_read) {
		perror("Error loading file");
	}
	
	return (bytes_read > 0);
}

int mfstream::getByte(char &nextByte) {

	if ( Input_buffer_pos < Input_buffer_size ) {
		unsigned char p = Input_buffer[Input_buffer_pos]; 
		Input_buffer_pos++;
		nextByte = (char) p;
		return 1;
	} else {
		return 0;
	}
}

int mfstream::ungetByte(char byte) {

	if ( Input_buffer_pos  > 0 ) {
		Input_buffer_pos--;
		Input_buffer[Input_buffer_pos] = byte;
		return 1;
	} else {
		return 0;
	}
}

int mfstream::writeByte(char nextByte) {

	if ( Output_buffer_pos >= Output_buffer_size ) {
		flushOutputBuffer();
	}
	
	Output_buffer[Output_buffer_pos] = nextByte;
	Output_buffer_pos++;
	return 1;
}

int mfstream::writeString( char* str ) {
	int i = 0;

	for ( i=0; i < (int) strlen(str); i++ ) {
		writeByte( str[i] );
	}
	return i;
}

int mfstream::flushOutputBuffer() {
	if ( Output_buffer_pos ) {
		fwrite( Output_buffer, 1, Output_buffer_pos, Output_file_ptr );
	}
	Output_buffer_pos = 0;
	return ferror(Output_file_ptr);
}

mfstream::~mfstream() {
	delete[] Input_buffer;
	delete[] Input_file_name;
	delete[] Output_file_name;
	flushOutputBuffer();
	fclose(Input_file_ptr);
	fclose(Output_file_ptr);
	delete[] Output_buffer;
}

int fileCopy( char* source, char* dest ) {
	if ( fopen( source, "r" ) && fopen( dest, "w" ) ) {
		mfstream* foo = new mfstream( source, dest );
		char c = 0;
		while ( foo->getByte( c ) ) {
			foo->writeByte(c);
		}
		delete foo;
		return 1;
	} else {
		printf( "ERROR: Unable to copy '%s' to '%s'\n", source, dest );
		return 0;
	}
}

#ifdef _DEBUG_MFS

// simple testing method. performs simple file copy ( a.out <source> ) 
// and produces a copy with ~ extension
int main(int argc, char* argv[] ) {

	fileCopy( argv[1], argv[2] );
	printf("Have a nice day!\n");
}

#endif

